use super::datatypes::Request;

impl Request {
    /// Sets the `mime_type` of this [`Request`] from the headers.
    ///
    /// The mime_type can remain none if there isn't a `Content-Type: ` header
    ///
    /// # Example
    /// ```
    /// use http::{
    ///     handling::{methods::Method, request::Request},
    ///     utils::{mime::Mime, urlencoded::UrlEncodeData},
    /// };
    ///
    /// let mut request = Request {
    ///     uri: UrlEncodeData::from_encoded("thing").unwrap(),
    ///     headers: vec![
    ///         "GET / 23".to_string(),
    ///         "SDF:LKJSD:F".to_string(),
    ///         "Content-Type: text/plain; charset=UTF-8".to_string(),
    ///         "SDF".to_string(),
    ///     ],
    ///     method: Method::Get,
    ///     cookies: None,
    ///     mime_type: None,
    /// };
    /// let mut wrong = Request {
    ///     uri: UrlEncodeData::from_encoded("thing").unwrap(),
    ///     headers: vec![
    ///         "GET / 23".to_string(),
    ///         "SDF:LKJSD:F".to_string(),
    ///         "SDF".to_string(),
    ///     ],
    ///     method: Method::Get,
    ///     cookies: None,
    ///     mime_type: None,
    /// };
    /// request.mime_from_headers();
    /// wrong.mime_from_headers();
    /// assert_eq!(None, wrong.mime_type);
    /// assert_eq!((Mime::TextPlain, " charset=UTF-8".to_string()), request.mime_type.unwrap());
    /// ```
    /// # Panics
    /// No Panics
    pub fn mime_from_headers(&mut self) {
        let Some(content_type_header) = self.headers
            .iter()
            .find(|header| header.starts_with("Content-Type: ")) else {
            return;
        };
        let content_type_string: &str = content_type_header
            .strip_prefix("Content-Type: ")
            .unwrap()
            .trim();

        self.mime_type = if let Some(content_type) = content_type_string.split_once(';') {
            let Ok(mime) = content_type.0.trim().parse() else {
                return;
            };
            Some((mime, content_type.1.to_owned()))
        } else {
            let Ok(mime) = content_type_string.parse() else {
                    return;
                };
            Some((mime, "".to_owned()))
        };
    }
}

#[cfg(test)]
mod test {
    use crate::{
        handling::{methods::Method, request::Request},
        utils::{mime::Mime, urlencoded::UrlEncodeData},
    };

    #[test]
    pub fn test_mime_parse_from_header_vec() {
        let mut request = Request {
            uri: UrlEncodeData::from_raw("thing"),
            headers: vec![
                "GET / 23".to_string(),
                "SDF:LKJSD:F".to_string(),
                "Content-Type: text/plain".to_string(),
                "SDF".to_string(),
            ],
            method: Method::Get,
            cookies: None,
            mime_type: None,
        };

        let mut wrong = Request {
            uri: UrlEncodeData::from_raw("thing"),
            headers: vec![
                "GET / 23".to_string(),
                "SDF:LKJSD:F".to_string(),
                "SDF".to_string(),
            ],
            method: Method::Get,
            cookies: None,
            mime_type: None,
        };
        request.mime_from_headers();
        wrong.mime_from_headers();
        assert_eq!(None, wrong.mime_type);
        assert_eq!((Mime::TextPlain, "".to_owned()), request.mime_type.unwrap());
    }
}
