use std::time::Duration;

use crate::utils::urlencoded::EnCodable;

use super::{Cookie, SameSite};

/// Builder wrapper for a Cookie
///
/// # Example
/// ```
/// use http::handling::response::Cookie;
/// use http::handling::response::CookieBuilder;
///
/// let cookie = CookieBuilder::build("name", "value").path("/").finish();
/// ```
pub struct CookieBuilder {
    /// Cookie under the hood
    inner: Cookie,
}

impl CookieBuilder {
    /// Builds a basic CookieBuilder from a name and a value
    pub fn build(name: &str, value: &str) -> Self {
        CookieBuilder {
            inner: Cookie {
                cookie_string: None,
                name: name.encode(),
                value: value.encode(),
                max_age: None,
                domain: None,
                path: None,
                secure: false,
                http_only: false,
                same_site: None,
                expires: None,
                partitioned: false,
            },
        }
    }
    pub fn finish(self) -> Cookie {
        self.inner
    }
    pub fn max_age(mut self, duration: Duration) -> Self {
        self.inner.max_age = Some(duration);
        self
    }
    pub fn domain(mut self, domain: &str) -> Self {
        self.inner.domain = Some(domain.encode());
        self
    }
    pub fn path(mut self, path: &str) -> Self {
        self.inner.path = Some(path.encode());
        self
    }
    pub fn secure(mut self, secure: bool) -> Self {
        self.inner.secure = secure;
        self
    }
    pub fn http_only(mut self, http_only: bool) -> Self {
        self.inner.http_only = http_only;
        self
    }
    pub fn same_site(mut self, same_site: SameSite) -> Self {
        self.inner.same_site = Some(same_site);
        self
    }
    pub fn expires(mut self, expire: &str) -> Self {
        self.inner.expires = Some(expire.encode());
        self
    }
    pub fn partitioned(mut self, partitioned: bool) -> Self {
        self.inner.partitioned = partitioned;
        self
    }
    pub fn name(mut self, name: &str) -> Self {
        self.inner.name = name.encode();
        self
    }
    pub fn value(mut self, value: &str) -> Self {
        self.inner.value = value.encode();
        self
    }
}
